from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.task_summary import TaskSummary


T = TypeVar("T", bound="TaskSummaryBatchRequest")


@attr.s(auto_attribs=True)
class TaskSummaryBatchRequest:
    """
    Attributes:
        items (List['TaskSummary']):
        ignore_duplicates (Union[Unset, bool]):
    """

    items: List["TaskSummary"]
    ignore_duplicates: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        items = []
        for items_item_data in self.items:
            items_item = items_item_data.to_dict()

            items.append(items_item)

        ignore_duplicates = self.ignore_duplicates

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "items": items,
            }
        )
        if ignore_duplicates is not UNSET:
            field_dict["ignoreDuplicates"] = ignore_duplicates

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.task_summary import TaskSummary

        d = src_dict.copy()
        items = []
        _items = d.pop("items")
        for items_item_data in _items:
            items_item = TaskSummary.from_dict(items_item_data)

            items.append(items_item)

        ignore_duplicates = d.pop("ignoreDuplicates", UNSET)

        task_summary_batch_request = cls(
            items=items,
            ignore_duplicates=ignore_duplicates,
        )

        task_summary_batch_request.additional_properties = d
        return task_summary_batch_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
